// ArduCAM Mini demo (C)2017 Lee
// Web: http://www.ArduCAM.com
// This program is a demo of how to use most of the functions
// This demo was made for ArduCAM_Mini platform.
// It needs to be used in combination with PC software.
// It can take photo continuously as video streaming.
// The demo support low power mode.
// The demo sketch will do the following tasks:
// 1. Set the camera to JPEG output mode.
// 2. Read data from Serial port and deal with it
// 3. If receive 0x00-0x08,the resolution will be changed.
// 4. If receive 0x10,camera will capture a JPEG photo and buffer the image to FIFO.Then write datas to Serial port.
// This program requires the ArduCAM V4.0.0 (or later) library and ArduCAM_Mini 2MP or 5MP
// and use Arduino IDE 1.6.8 compiler or above
#include <Wire.h>
#include <ArduCAM.h>
#include <SPI.h>
#include "memorysaver.h"

//This demo can only work on OV2640_MINI_2MP or OV5642_MINI_5MP or OV5642_MINI_5MP_BIT_ROTATION_FIXED platform.
#if !(defined OV2640_MINI_2MP)
  #error Please select the hardware platform and camera module in the ../libraries/ArduCAM/memorysaver.h file
#endif
const int CS = 10;
#if defined (OV2640_MINI_2MP)
  ArduCAM myCAM( OV2640, CS );
#elif defined (OV3640_MINI_3MP)
  ArduCAM myCAM( OV3640, CS );
#else
  ArduCAM myCAM( OV5642, CS );
#endif
uint8_t read_fifo_burst(ArduCAM myCAM);
void setup() {
uint8_t vid, pid;
uint8_t temp;

  Wire.begin();
  Serial.begin(115200);

// Serial.println(F("ACK CMD ArduCAM Start!")); 
// set the SPI_CS as an output:
pinMode(CS, OUTPUT);
// initialize SPI:
SPI.begin(); 
while(1){
  //Check if the ArduCAM SPI bus is OK
  myCAM.write_reg(ARDUCHIP_TEST1, 0x55);
  temp = myCAM.read_reg(ARDUCHIP_TEST1);
  if (temp != 0x55){
    Serial.println(F("ACK CMD SPI interface Error!"));
    delay(1000);continue;
  }else{
//    Serial.println(F("ACK CMD SPI interface OK."));
break;
  }
}
myCAM.clear_bit(ARDUCHIP_GPIO, GPIO_PWDN_MASK);
delay(100);
#if defined (OV2640_MINI_2MP)
  while(1){
    //Check if the camera module type is OV2640
    myCAM.wrSensorReg8_8(0xff, 0x01);
    myCAM.rdSensorReg8_8(OV2640_CHIPID_HIGH, &vid);
    myCAM.rdSensorReg8_8(OV2640_CHIPID_LOW, &pid);

// Serial.print(vid);
// Serial.print(" - ");
// Serial.println(pid);

    if ((vid != 0x26 ) && (( pid != 0x41 ) || ( pid != 0x42 ))){
      Serial.println(F("ACK CMD Can't find OV2640 module!"));
      delay(1000);continue;
    }
    else{
   //   Serial.println(F("ACK CMD OV2640 detected."));
   break;
    } 
  }
#endif
//Change to JPEG capture mode and initialize the OV5642 module	
myCAM.set_format(JPEG);
myCAM.InitCAM();

  myCAM.OV2640_set_JPEG_size(OV2640_320x240);

myCAM.clear_fifo_flag();
#if !(defined (OV2640_MINI_2MP))
  myCAM.write_reg(ARDUCHIP_FRAMES,0x00);
#endif
// myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
}

void loop() {
uint8_t temp = 0xff, temp_last = 0;
uint8_t start_capture = 0;
temp = Serial.read();
// Serial.println(temp);
switch(temp)
{
  case '0':
  temp = 0xff;
 // myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  #if defined (OV2640_MINI_2MP) 
    myCAM.OV2640_set_JPEG_size(OV2640_160x120);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_160x120")); 
 #elif defined (OV3640_MINI_3MP)
    myCAM.OV3640_set_JPEG_size(OV3640_176x144);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_160x120"));
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_320x240);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_320x240"));
  #endif 
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK); 
  break;
  case '1':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  #if defined (OV2640_MINI_2MP)
    myCAM.OV2640_set_JPEG_size(OV2640_176x144);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_176x144"));
  #elif defined (OV3640_MINI_3MP)
      myCAM.OV3640_set_JPEG_size(OV3640_320x240);delay(1000);
      Serial.println(F("ACK CMD switch to OV3640_320x240"));       
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_640x480);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_640x480"));
  #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  break;
  
  case '2':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  #if defined (OV2640_MINI_2MP)   
    myCAM.OV2640_set_JPEG_size(OV2640_320x240);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_320x240")); 
  #elif defined (OV3640_MINI_3MP)
      myCAM.OV3640_set_JPEG_size(OV3640_352x288);delay(1000);
      Serial.println(F("ACK CMD switch to OV3640_352x288"));   
  #else   
    myCAM.OV5642_set_JPEG_size(OV5642_1024x768);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_1024x768"));  
  #endif 
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);  
  break;
  
  case '3':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  #if defined (OV2640_MINI_2MP)      
    myCAM.OV2640_set_JPEG_size(OV2640_352x288);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_352x288")); 
  #elif defined (OV3640_MINI_3MP)
      myCAM.OV3640_set_JPEG_size(OV3640_640x480);delay(1000);
      Serial.println(F("ACK CMD switch to OV3640_640x480"));     
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_1280x960);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_1280x960"));
  #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  break;
  
  case '4':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);   
  #if defined (OV2640_MINI_2MP)        
    myCAM.OV2640_set_JPEG_size(OV2640_640x480);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_640x480"));
 #elif defined (OV3640_MINI_3MP)
      myCAM.OV3640_set_JPEG_size(OV3640_800x600);delay(1000);
      Serial.println(F("ACK CMD switch to OV3640_800x600")); 
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_1600x1200);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_1600x1200"));
  #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK); 
  break;
  case '5':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK); 
  #if defined (OV2640_MINI_2MP)
    myCAM.OV2640_set_JPEG_size(OV2640_800x600);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_800x600")); 
 #elif defined (OV3640_MINI_3MP)
      myCAM.OV3640_set_JPEG_size(OV3640_1024x768);delay(1000);
      Serial.println(F("ACK CMD switch to OV3640_1024x768")); 
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_2048x1536);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_2048x1536"));
  #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  break;
  
  case '6':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);  
  #if defined (OV2640_MINI_2MP)  
    myCAM.OV2640_set_JPEG_size(OV2640_1024x768);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_1024x768")); 
 #elif defined (OV3640_MINI_3MP)
    myCAM.OV3640_set_JPEG_size(OV3640_1280x960);delay(1000);
    Serial.println(F("ACK CMD switch to OV3640_1280x960"));       
  #else
    myCAM.OV5642_set_JPEG_size(OV5642_2592x1944);delay(1000);
    Serial.println(F("ACK CMD switch to OV5642_2592x1944"));
  #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK); 
  break;
  case '7':
  temp = 0xff;
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  #if defined (OV2640_MINI_2MP)
    myCAM.OV2640_set_JPEG_size(OV2640_1280x1024);delay(1000);
    Serial.println(F("ACK CMD switch to OV2640_1280x1024"));
  #else
    myCAM.OV3640_set_JPEG_size(OV3640_1600x1200);delay(1000);
    Serial.println(F("ACK CMD switch to OV3640_1600x1200"));
   #endif
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  break;
  case '8':
  temp = 0xff;
   #if defined (OV2640_MINI_2MP)
     myCAM.OV2640_set_JPEG_size(OV2640_1600x1200);delay(1000);
     Serial.println(F("ACK CMD switch to OV2640_1600x1200"));
   #else
     myCAM.OV3640_set_JPEG_size(OV3640_2048x1536);delay(1000);
     Serial.println(F("ACK CMD switch to OV3640_2048x1536"));
   #endif
   myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  break;
  case '9':
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  myCAMSaveToSDFile();
  
  temp = 0xff;
 /*
  start_capture = 1;  
 // Serial.println(F("ACK CMD CAM start single shot."));
  myCAM.clear_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
  */
  delay(800);
  break;
  default:
  break;
}
/*
if(start_capture == 1)
{
  myCAM.flush_fifo();
  myCAM.clear_fifo_flag();	 
  //Start capture
  myCAM.start_capture();
  start_capture = 0;
}
if(myCAM.get_bit(ARDUCHIP_TRIG, CAP_DONE_MASK))
{
  myCAM.set_bit(ARDUCHIP_GPIO,GPIO_PWDN_MASK);
 // Serial.println(F("ACK CMD CAM Capture Done."));
  temp = 0;
//  Serial.println(F("ACK IMG"));
  while( (temp != 0xD9) | (temp_last != 0xFF) )
  {
    temp_last = temp;
    temp = myCAM.read_fifo();
    Serial.write(temp);
   // delayMicroseconds(10);
  }
  //Clear the capture done flag 
  myCAM.clear_fifo_flag(); 
}
*/
}


void myCAMSaveToSDFile(){
char str[8];
byte buf[256];
static int i = 0;
static int k = 0;
uint8_t temp = 0,temp_last=0;
uint32_t length = 0;
bool is_header = false;
// File outFile;
//Flush the FIFO
myCAM.flush_fifo();
//Clear the capture done flag
myCAM.clear_fifo_flag();
//Start capture
myCAM.start_capture();
//Serial.println(F("start Capture"));
while(!myCAM.get_bit(ARDUCHIP_TRIG , CAP_DONE_MASK));
//Serial.println(F("Capture Done."));  
length = myCAM.read_fifo_length();
//Serial.print(F("The fifo length is :"));
//Serial.println(length, DEC);
if (length >= MAX_FIFO_SIZE) //384K
{
  // Serial.println(F("Over size."));
  return ;
}
if (length == 0 ) //0 kb
{
 // Serial.println(F("Size is 0."));
  return ;
}
//Construct a file name
/*
k = k + 1;
itoa(k, str, 10);
strcat(str, ".jpg");
//Open the new file
outFile = SD.open(str, O_WRITE | O_CREAT | O_TRUNC);
if(!outFile){
  Serial.println(F("File open faild"));
  return;
}
*/
myCAM.CS_LOW();
myCAM.set_fifo_burst();
while ( length-- )
{
  temp_last = temp;
  temp =  SPI.transfer(0x00);
  //Read JPEG data from FIFO
  if ( (temp == 0xD9) && (temp_last == 0xFF) ) //If find the end ,break while,
  {
    buf[i++] = temp;  //save the last  0XD9     
    //Write the remain bytes in the buffer
    myCAM.CS_HIGH();
   // outFile.write(buf, i);
    Serial.write(buf, i);    
    //Close the file
   // outFile.close();
   // Serial.println(F("Image save OK."));
    is_header = false;
    i = 0;
  }  
  if (is_header == true)
  { 
    //Write image data to buffer if not full
    if (i < 256)
    buf[i++] = temp;
    else
    {
      //Write 256 bytes image data to file
      myCAM.CS_HIGH();
      // outFile.write(buf, 256);
      Serial.write(buf, 256);   
      i = 0;
      buf[i++] = temp;
      myCAM.CS_LOW();
      myCAM.set_fifo_burst();
    }        
  }
  else if ((temp == 0xD8) & (temp_last == 0xFF))
  {
    is_header = true;
    buf[i++] = temp_last;
    buf[i++] = temp;   
  } 
} 
}



